﻿using Borepin.Base;
using Borepin.Model;
using Borepin.Service.BFFH;
using Borepin.Service.Connections;
using Prism.Commands;
using Prism.Navigation;
using Prism.Services;
using System;
using System.Threading.Tasks;
using System.Windows.Input;

namespace Borepin.PageModel.AddServerProcess
{
    public class LoginPasswordPageModel : PageModelBase
    {
        #region Private Properties
        private readonly IBFFHService _BFFHService;
        private readonly IConnectionService _ConnectionService;
        private readonly IPageDialogService _PageDialogService;
        #endregion

        #region Constructors
        public LoginPasswordPageModel(INavigationService navigationService, IBFFHService bffhService, IConnectionService connectionService, IPageDialogService pageDialogService) : base(navigationService)
        {
            _BFFHService = bffhService;
            _ConnectionService = connectionService;
            _PageDialogService = pageDialogService;


            AuthenticateCommand = new DelegateCommand(async () => await AuthenticateCommandExecuted());

            Task.Run(LoadData);
        }
        #endregion

        #region LoadData
        public override Task LoadData()
        {
            IsBusy = false;

            return Task.CompletedTask;
        }
        #endregion

        #region Properties
        private string _Username;
        public string Username
        {
            get => _Username;
            set => SetProperty(ref _Username, value);
        }

        private string _Password;
        public string Password
        {
            get => _Password;
            set => SetProperty(ref _Password, value);
        }
        #endregion

        #region Commands
        private ICommand _AuthenticateCommand;
        public ICommand AuthenticateCommand
        {
            get => _AuthenticateCommand;
            set => SetProperty(ref _AuthenticateCommand, value);
        }

        private async Task AuthenticateCommandExecuted()
        {
            IsBusy = true;

            Connection connection_update = _BFFHService.ActiveConnection;

            connection_update.Username = Username;

            if(!await _BFFHService.Authenticate(connection_update, Password))
            {
                await _PageDialogService.DisplayAlertAsync("Connection failed", "Unable to authenticate to server.", "Ok");

                IsBusy = false;
                return;
            }

            try
            {
                await _ConnectionService.AddConnection(_BFFHService.ActiveConnection);
            }
            catch (ArgumentException)
            {
                // Could be better catched
            }

            await _ConnectionService.LogConnect(_BFFHService.ActiveConnection);

            var result = await _NavigationService.NavigateAsync("/MainPage/NavigationPage/MachineListPage");

            if (!result.Success)
            {
                System.Diagnostics.Debugger.Break();
            }
        }
        #endregion

        #region INavigationAware
        public override void OnNavigatedFrom(INavigationParameters parameters)
        {

        }

        public override void OnNavigatedTo(INavigationParameters parameters)
        {

        }
        #endregion
    }
}